%perform iterative simualted annealing IB algorithm combined with Tishby et
%al. steps.
% pertub_probS is a vector of pertubation probabilties (this determines the
% step size in the annealing algorithm).
% ITER is the number of iteration in each step.
% temperatureS is a vector of temperature fot the annealing process.
% what_to_do is a vector that it's length determine how much runs with
% different intial conditions to run the entire process. 
% 1 in this vecotr means keep the original PCX0 and PTY0 as initial
% conditions.
% 2 means random permutation of PCX0 and PTY0 as initial conditions.
% 3 means random initial conditions (of size of PCX0 and PTY0). 
function [bbPCX,bbPTY,bbPTC,bbPYC,bbL,Ls]=IBLIB_do_annealing(x,y,PCX0,PTY0,PYX,mybeta,ITER,pertub_probS,temperatureS,what_to_do,VERBOSE,NAMESX)
func=@IBLIB_do_IB_iteration; % this is the iternal function handler to work with (does some iterations of the Tishby et al. algorithm)
CONST_ORIG=1; % initial conditions are PCX0, PTY0
CONST_PERM=2; % initial conditions are random permutations of PCX0, PTY0
CONST_PERM_D=2.5; % initial conditions are random permutations to deterministic clusters
CONST_RAND=3; % initial conditions are random distrebutions
Ls=[]; %list of Lagrangian values

% Keep track of best over all
bbL=9999999999; 
bbPCX=[];
bbPTY=[];
bbPTC=[];
bbPYC=[];

cnt=0; %count all iterations...

% Initialize default values
if isempty(what_to_do)
    what_to_do=[CONST_ORIG,CONST_PERM,CONST_PERM,CONST_PERM,CONST_RAND,CONST_RAND];
    %what_to_do=[1,2,2,2,3,3];
end
if isempty(temperatureS)
        temperatureS=[1e-10,100,1e-1,1,10,100 ,1e-5];
end
if isempty(pertub_probS)
        pertub_probS=[0.01 0.9 0.3 0.5 1 0.7 0.1];
end

% scan all what_to_dos
for mywhat=what_to_do,

    % set initial values
    if mywhat==CONST_ORIG
        PCX=PCX0;
        PTY=PTY0;
    end
    if mywhat==CONST_PERM
        PCX=PCX0(:,randperm(size(PCX0,2)));
        PTY=PTY0(:,randperm(size(PTY0,2)));
        
    end
    if mywhat==CONST_PERM_D
        PCX=zeros(KS(k),length(x));
        PTY=zeros(SS(k),length(y));
        for jjj=1:length(x)   % random assignment to clusters
            PCX(mod(jjj-1,size(PCX,1))+1,jjj)=1;
        end
        for jj=1:length(y) % random assignment to clusters
            PTY(mod(jjj,size(PTY,1))+1,jjj)=1;
        end
        PCX=PCX(:,randperm(size(PCX,2)));
        PTY=PTY(:,randperm(size(PTY,2)));
    end
    
    if mywhat==CONST_RAND
        PCX=rand(size(PCX0));
        PCX=PCX./repmat(sum(PCX),size(PCX,1),1);
        PTY=rand(size(PTY0));
        PTY=PTY./repmat(sum(PTY),size(PTY,1),1);
    end

    bL=9999999;
    bPCX=PCX;
    bPTY=PTY;
    if (length(x)==length(y))
        bPTC=PTY*PYX*((PCX').*repmat(x,[1 size(PCX,1)]).*repmat((1./((PCX*x)+eps))' , [size(PCX,2) 1]));
    else
        bPTC=0;
    end
    bPYC=PYX*((PCX').*repmat(x,[1 size(PCX,1)]).*repmat((1./((PCX*x)+eps))' , [size(PCX,2) 1]));

    
    ITERsmall=3;% The current solution iterates 3 times before considering pertubation.
    ITERpertub=1; % The pertubed solution is iterated once before compared with the current solution.

    % scan temperatures and  pertub probabilities
    for temperature=temperatureS,
        for pertub_prob=pertub_probS,
            if VERBOSE>=3
                 display(sprintf('what_to_do=%d temperature= %g pertub_prob=%g bestL=%g bestbestL=%g',mywhat,temperature,pertub_prob,bL,bbL));
            end
            iter=1;
            while (iter<=ITER)
                cnt=cnt+1;
                
                % Run current solution
                [PCX,PTY,PTC,PYC,L]=func(x,y,PCX,PTY,PYX,mybeta,ITERsmall);
                iter=iter+ITERsmall;

                % Pertub solution and run once
                 Z=rand(size(PCX));Z=Z./repmat(sum(Z),size(Z,1),1);
                 PCX1=(1-pertub_prob)*PCX + pertub_prob*Z;
                 PCX1=PCX1./repmat(sum(PCX1),size(PCX1,1),1);
                 if (pertub_prob>0.15)&&(pertub_prob<=0.7)
                    nn=randi(size(PCX1,2));
                    PCX1(:,nn)=PCX1(randperm(size(PCX1,1)),nn);
                 end
                 

                 Z=rand(size(PTY));Z=Z./repmat(sum(Z),size(Z,1),1);
                 PTY1=(1-pertub_prob)*PTY + pertub_prob*Z;
                 PTY1=PTY1./repmat(sum(PTY1),size(PTY1,1),1);
                 
                if (pertub_prob>0.15)&&(pertub_prob<=0.7)
                    nn=randi(size(PTY1,2));
                    PTY1(:,nn)=PTY1(randperm(size(PTY1,1)),nn);
                 end
    

                 [PCX1,PTY1,PTC1,PYC1,L1]=func(x,y,PCX1,PTY1,PYX,mybeta,ITERpertub);
                 iter=iter+ITERpertub;

                 % Local search move 
                 if L1<=L
                     PCX=PCX1;
                     PTY=PTY1;
                     PTC=PTC1;
                     PYC=PYC1;
                     L=L1;
                 end
                 
                 if(mod(cnt,1000)==999) && VERBOSE>=0.5
                     display(sprintf('.......intermidiate results:now in iteration %d: what_to_do=%d temperature= %g pertub_prob=%g bestL=%g bestbestL=%g',cnt,mywhat,temperature,pertub_prob,bL,bbL));
                 end


                 if L<bL
                     bPCX=PCX;
                     bPTY=PTY;
                     bPTC=PTC;
                     bPYC=PYC;
                     bL=L;
                     if VERBOSE>=2
                         display(sprintf('what_to_do=%d temperature= %g pertub_prob=%g bestL=%g bestbestL=%g iter=%d',mywhat,temperature,pertub_prob,bL,bbL,iter));
                     end
                 end
                 
                 if L<bbL
                     bbPCX=PCX;
                     bbPTY=PTY;
                     bbPTC=PTC;
                     bbPYC=PYC;
                     bbL=L;
                     if VERBOSE>=1
                          display(sprintf('what_to_do=%d temperature= %g pertub_prob=%g bestL=%g bestbestL=%g iter=%d',mywhat,temperature,pertub_prob,bL,bbL,iter));
                     end
                     if VERBOSE>=4
                         [msgA,~]=LIB_print_clusters(PCX,x,NAMESX,0.1,0);
                         display(sprintf('new PCX: %s',msgA));
                     end
                 end
               
                 if L1>L
                     if rand(1)<exp( (L-L1)/temperature ) %simulated annealing  step
                        PCX=PCX1;
                        PTY=PTY1;
                        PTC=PTC1;
                        PYC=PYC1;
                        L=L1;

                     end
                 end
                 Ls=[Ls,L];

            end %while (iter<=ITER)
        end %pertub_prob=pertub_probS,
    end %temperature=temperatureS,
end

