% this function reads predetermined and joint histograms. joint histograms
% has the format that each x-token is in a row and each column correspond
% to y-tokens (the input data is therefore a NY columsn by NX rows).
% The first columns are the names of X tokens. 
% the first row are the names of Y tokens.
% the predetermined first columns has to be the same as the X tokens.
% the other columsn of this input file are pre-determined categories.
% the name of the category is in the first row.
% each column gives the label of each token. 
%%% here are examples of input files:
%
%
% joint-histogram.csv
%
% FROMTO,I   ,ii   ,iii ,IV  ,V   ,vi  ,viio
% I     ,0   ,392  ,14  ,582 ,896 ,262 ,106
% ii    ,12  ,0    ,15  ,5   ,674 ,17  ,182
% iii   ,13  ,6    ,0   ,44  ,12  ,32  ,3
% IV    ,239 ,134  ,16  ,0   ,380 ,13  ,173
% V     ,2099,25   ,46  ,93  ,0   ,211 ,2
% vi    ,96  ,206  ,25  ,95  ,172 ,0   ,36
% viio  ,499 ,1    ,4   ,9   ,24  ,20  ,0
%
% predetermined-categories.csv
%
% CHORD ,Mmd        ,TSD
% I     ,Major      ,Tonic
% ii    ,minor      ,Subdominant
% iii   ,minor      ,Tonic
% IV    ,Major      ,Subdominant
% V     ,Major      ,Dominant
% vi    ,minor      ,Tonic
% viio  ,diminished ,Dominant

function [msg,NAMESX,NAMESY,NUM,NAMESZ,CATZ_NAMEs,CATZs,NX,NY,NZ,pyx]=IBLIB_read_data_files(fname_data,fname_pre_determined_clusters)
msg='';NAMESX={};NAMESY={};NUM=nan;NAMESZ={};CATZ_NAMEs={};CATZs={};NX=-1;NY=-1;NZ=-1;pyx=nan;
display(sprintf('read histogram file...'));
try
    FID=fopen(fname_data);
catch err
    msg=sprintf('error reading file %s: file name is wrong or file can not be read',fname_data);
    display(sprintf('%s',msg));
    return;
end
if FID==-1
    msg=sprintf('error reading file %s: file name is wrong or file can not be read',fname_data);
    display(sprintf('%s',msg));
    return;
end

str = fscanf(FID,'%s,%s',[1,inf]);
dat = regexp(str, ',', 'split');
NAMESX=cell(1,2); %this will grow...
NAMESY=cell(1,length(dat)-1);
for I=2:length(dat),
    NAMESY(I-1)=dat(I);
end
NX=0;
NY=length(NAMESY);
NUM=nan(NY,2); % this will be large
str='start';
while ~isempty(str)
    
    %     for J=1:NX,
    str = fscanf(FID,'%s,%s',[1,inf]);
    dat = regexp(str, ',', 'split');
    if length(dat)>1
        NX=NX+1;
        NAMESX{NX}=dat{1};
        if length(dat)~=(NY+1)
            fclose(FID);
            msg=sprintf('error in reading histogram file line %d has inproper length (%d)\n check if names of tokens have spaces if so replace them with another character',J,length(dat)-1);
            
            return;
        end
        for I=1:NY,
            NUM(I,NX)=str2num(dat{I+1});
        end
    end
end

if NX<=2
    fclose(FID);
    msg=sprintf('error in reading histogram file it seems that there are no rows to read (not enought raws in input matrix)');
    
    return;
end

display('ok reading histogram file');
fclose(FID);

if (sum(sum(isnan(NUM)))>0) ||(sum(sum((NUM<0)))>0)
    msg=sprintf('bad values in histogram file');
    return;
end


if ~isempty(fname_pre_determined_clusters)
    display(sprintf('read histogram file...'));
    try
        FID=fopen(fname_pre_determined_clusters);
    catch err
        msg=sprintf('no predetermined cluster file %s',fname_pre_determined_clusters);
        display(sprintf('%s',msg));
    end
    if FID==-1
        fname_pre_determined_clusters=[];
    end
end

if isempty(fname_pre_determined_clusters)
    sprintf('no/bad predetermined cluster file continue without it!');
end


if ~isempty(fname_pre_determined_clusters)
    display(sprintf('proces pre-detrmined categories...'));
    str = fscanf(FID,'%s,%s',[1,inf]);
    dat = regexp(str, ',', 'split');
    
    CATZ_NAMEs=cell(1,length(dat)-1);
    for I=2:length(dat),
        CATZ_NAMEs(I-1)=dat(I);
    end
    NZ=length(dat)-1;
    
    CATZs=cell(NZ,1);
    for J=1:NX,
        str = fscanf(FID,'%s,%s',[1,inf]);
        dat = regexp(str, ',', 'split');
        NAMESZ{J}=dat{1};
        if length(dat)~=(NZ+1)
            fclose(FID);
            msg=sprintf('error in reading category file line %d has inproper length (%d)\n check if names of tokens have spaces if so replace them with another character',J,length(dat)-1);
            display(sprintf('%s',msg));
            return;
        end
        for I=1:NZ,
            CATZs{I}{J}=dat{I+1};
        end
    end
    display('ok reading categories file');
    fclose(FID);
    
    %     for I=1:NZ,
    %         CATZs{I}=TXT2(2:end,I+1);
    %         CATZ_NAMEs{I}=TXT2{1,I+1};
    %     end
else
    NZ=0;
end


display(sprintf('finished reading data!'));
%NX=size(NUM,1);
%NY=size(NUM,1);
%NAMESX=TXT(2:(NX+1),1)';
%NAMESY=TXT(1,2:(NY+1));
% for I=1:NX,
%     if ~(strcmp(NAMESX(I),NAMESY(I)));
%         msg=sprintf('error in joint histogram line %s  (line number %d) doesn''t match line %s : token on raws and columsn should be identical',...
%             NAMESX{I},I,NAMESY{I});
%         display(sprintf('problem in input: %s',msg));
%         return
%     end
% end
if ~isempty(fname_pre_determined_clusters)
    for I=1:NX,
        if ~(strcmp(NAMESX(I),NAMESZ(I)))
            msg=sprintf('error in names of tokens: in token %s  (line number %d) doesn''t match histogram token line %s : token on predetermined clusters should be identical to hisogram tokens',...
                NAMESX{I},I,NAMESZ{I});
            display(sprintf('problem in input: %s',msg));
            return
        end
        
    end
end
pyx=(NUM);


% for I=1:NX,  % don't do it!!!
%     pyx(I,I)=0;
% end

% % if this is histogram of values and not probabilties remove low count
% % tokens, else ignore it (but you have to make sure that you don't have
% % zero enteries...
% 
    display(sprintf('remove 0 count enteries...'));


    %     NAMESY=NAMESY(pos); %don't do that! we only concern with 0
    %     marginals in x.
   isxy=true; %if the names are identical remove both rows and columns otherwise remove only 0 marginal x.
    if length(NAMESX)==length(NAMESY)
        for I=1:length(NAMESX)
            if ~strcmp(NAMESX{I},NAMESY{I})
                isxy=false;
                break;
            end
        end
    else
        isxy=false;
    end
    if isxy
        if (sum(sum((round(pyx)~=pyx)))==0) % so this is not a distrebution file but histograms
%             pos=(sum(pyx)>=5)&((sum(pyx,2)>=5)'); % only freqeunt chords are selected threshold of 10 appearences is required
            pos=(sum(pyx)>0);
        else
            pos=(sum(pyx)>0);
        end
        
        pyx=pyx(pos,pos);
        NAMESX=NAMESX(pos);
        NAMESY=NAMESY(pos);
    else
        pos=(sum(pyx)>0);
        pyx=pyx(:,pos);
        NAMESX=NAMESX(pos);
    end

    
    if ~isempty(fname_pre_determined_clusters)
        for I=1:NZ,
            CATZs{I}=CATZs{I}(pos);
        end
    end
% end
%

NX=length(NAMESX);
NY=length(NAMESY);
assert(size(pyx,1)==NY);
assert(size(pyx,2)==NX);