% this script reads parameters from file (instead of setting them from
% MATLAB). The file suppose to be located in input\params.txt
% The file should look like:
%       
% 
% curve-iterations,20
% clusters-iterations,200
% ks,2,3,4,5,6,7
% betas,120,150,150,150,150,150
% beta-range,80,1.1
% beta-jump,0.5
% 
% note that the function do not accept changes in order and do not expect
% space or small deviation from the format above.
%
function [param,msg]=IBLIB_read_params_from_file(fname)
msg='';
display(sprintf('reading parameters from file...'));
FID=fopen(fname);
NP=6;
params=cell(NP,1);

if FID==-1
    display(sprintf('cant read parameter file %s ...',fname));
    msg=sprintf('cant read parameter file %s ...',fname);
    return
end
    
for I=1:NP,
 str = fscanf(FID,'%s,%s',[1,inf]);
 dat = regexp(str, ',', 'split');
 params{I}=dat;
end
fclose(FID);


POS=1;
if ~strcmp(params{POS}{1},'curve-iterations')
    msg=(sprintf('Error reding params file:first parameter should be curve-iterations'));
    return;
else
    ITER_infocurve=str2num(params{POS}{2});
    if isempty(ITER_infocurve)
        ITER_infocurve=20;
    end
    if (ITER_infocurve<1)||(ITER_infocurve>999999)||isnan(ITER_infocurve)
        msg=(sprintf('Error reding params file:bad curve-iterations %d',ITER_infocurve));
        return;
    end
end
param.ITER_infocurve=20;

POS=2;
if ~strcmp(params{POS}{1},'clusters-iterations')
    msg=(sprintf('Error reding params file:second parameter should be clusters-iterations'));
    return;
else
    ITER_clusters=str2num(params{POS}{2});
    if isempty(ITER_clusters)
        ITER_clusters=40;
    end
    if (ITER_clusters<1)||(ITER_clusters>999999)||isnan(ITER_clusters)
        msg=(sprintf('Error reding params file:bad clusters-iterations %d',ITER_clusters));
        return;
    end
end
param.ITER_clusters=ITER_clusters;
    
POS=3;
if ~(strcmp(params{POS}{1},'ks'))
    msg=(sprintf('Error reding params file:third line should be ks...'));
    return;
end

if (isempty(params{POS}{2}))
    KS=[];
    display(sprintf('ks were empty! not going to do clustering'));
else
    for J=2:length(params{POS})
        KS(J-1)=str2num(params{POS}{J});
        if (KS(J-1)<2)||(KS(J-1)>9999)||(isnan(KS(J-1)))
            msg=sprintf('Error reding params file:bad ks values at position %d (%g)',J-1,KS(J-1));
            return;
        end
    end
    display(sprintf('setting KS...'));
    KS
end
param.KS=KS;

POS=4;
if ~(strcmp(params{POS}{1},'betas'))
    msg=(sprintf('Error reding params file:4th line should be betas...'));
    return;
end
if isempty(params{POS}{2})
    KS_betaS=[];
    display(sprintf('ks were empty - setting to default'));
else
    for J=2:length(params{POS})
        KS_betaS(J-1)=str2num(params{POS}{J});
        if (KS_betaS(J-1)<2)||(KS_betaS(J-1)>9999)||(isnan(KS_betaS(J-1)))
            msg=sprintf('Error reding params file:bad ks values at position %d (%g)',J-1,KS_betaS(J-1));
            return;
        end
    end
    display(sprintf('setting betas of KS...'));
    KS_betaS
end
if length(KS)~=length(KS_betaS)
    msg=(sprintf('Error reding params file:KS and KS_betaS should have the same size!'));
    return;
end
param.KS_betaS=KS_betaS;

beta_range=[];
POS=5;
if ~strcmp(params{POS}{1},'beta-range')
    msg=(sprintf('Error reding params file:missing parameter beta-range in 5th line'));
    return;
else
    if isempty(params{POS}{2})
        display(sprintf('beta-range is empty - setting to default'));
        beta_range=[80 1.5];
    elseif length(params{POS}(2:end))~=2
        display(sprintf('beta-range is not proper - setting to default'));
        beta_range=[80 1.5];
    else
        beta_range=[str2num(params{POS}{2}),str2num(params{POS}{3})];
        beta_range=[max(beta_range),min(beta_range)]
    end
end
param.beta_range=beta_range;

POS=6;
if ~strcmp(params{POS}{1},'beta-jump')
    msg=(sprintf('Error reding params file:missing parameter beta-jump in 6th line'));
    return;
else
    if length(params{POS}(2))~=1
        display(sprintf('beta-jump is not proper - setting to default'));
        beta_jump=.1;
    else
        beta_jump=[abs(str2num(params{POS}{2}))]
    end
end
param.beta_jump=beta_jump;